#!/usr/bin/python
# -*- coding: utf-8 -*-

# Libervia: a Salut à Toi frontend
# Copyright (C) 2013  Emmanuel Gil Peyrot <linkmauve@linkmauve.fr>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from twisted.internet import defer
if defer.Deferred.debug:
    # if we are in debug mode, we want to use ipdb instead of pdb
    try:
        import ipdb
        import pdb
        pdb.set_trace = ipdb.set_trace
        pdb.post_mortem = ipdb.post_mortem
    except ImportError:
        pass

# XXX: We need to configure logs before any log method is used, so here is the best place.
from libervia.server.constants import Const as C
from sat.core import log_config
log_config.satConfigure(C.LOG_BACKEND_TWISTED, C)



from sat.core.i18n import _
from sat.tools.config import getConfig

from zope.interface import implements

from twisted.python import usage
from twisted.plugin import IPlugin
from twisted.application.service import IServiceMaker

from ConfigParser import SafeConfigParser, NoSectionError, NoOptionError
try:
    from libervia.server.server import Libervia
    opt_params = Libervia.OPT_PARAMETERS_BOTH
    cfg_params = Libervia.OPT_PARAMETERS_CFG
except (ImportError, SystemExit):
    # avoid raising an error when you call twisted and sat is not launched
    opt_params = []
    cfg_params = []


class Options(usage.Options):

    # optArgs is not really useful in our case, we need more than a flag
    optParameters = opt_params

    def __init__(self):
        """Read SàT configuration file in order to overwrite the hard-coded default values.

        Priority for the usage of the values is (from lowest to highest):
            - hard-coded default values
            - values from SàT configuration files
            - values passed on the command line
        """
        # If we do it the reading later: after the command line options have been parsed, there's no good way to know
        # if the  options values are the hard-coded ones or if they have been passed on the command line.

        # FIXME: must be refactored + code can be factorised with backend
        config = SafeConfigParser()
        config.read(C.CONFIG_FILES)
        for param in self.optParameters + cfg_params:
            name = param[0]
            try:
                value = getConfig(config, 'libervia', name)
                try:
                    param[2] = param[4](value)
                except IndexError: # the coerce method is optional
                    param[2] = value
            except (NoSectionError, NoOptionError):
                pass
        usage.Options.__init__(self)


class LiberviaMaker(object):
    implements(IServiceMaker, IPlugin)

    tapname = C.APP_NAME_FILE
    description = _(u'The web frontend of Salut à Toi').encode('utf-8')
    options = Options

    def makeService(self, options):
        return Libervia(**dict(options))  # get rid of the usage.Option overload


# affectation to some variable is necessary for twisted introspection to work
serviceMaker = LiberviaMaker()
